#include "header.h"

extern int debug;
extern int queued;
extern int module;
extern int capacity;
extern int blocksize;
extern int iosize;

void devinfo( char *dev )
{
  int res;
  /* int queue; */

  My_sg_scsi_id m_id;

  int tfd = open( dev, O_RDWR );

  res = ioctl(tfd, SG_GET_SCSI_ID, &m_id);
  if (res < 0) 
  {
    perror("ioctl() SG_GET_SCSI_ID");
    removeModule( module );
    exit(EXIT_FAILURE);
  }

  if ( debug ) printf( "Device properties\n"
                       "  Device type    = %d\n"
		       "  Commands / Lun = %hd\n"
		       "  Queue Depth    = %hd\n", 
		       m_id.scsi_type,
		       m_id.h_cmd_per_lun+1, 
		       m_id.d_queue_depth+1);

  /*
   * Queue is the lesser of Host queue for lun and
   * device queue
   *
  if( m_id.h_cmd_per_lun < m_id.d_queue_depth )
    queue = m_id.h_cmd_per_lun+1;
  else
    queue = m_id.d_queue_depth+1;

  *
   * Queue size if finally determined whether user
   * requests queueing. If he does, queue is the
   * lesser of requested size and above value.
   *
  if( queued > queue ) queued = queue;
  */

  /*
   * Get the capacity and reduce it by iosize/blocksize so
   * that the maximum lba requested can always fit a request
   */
  capacity = readcap( tfd, &blocksize ) - iosize/blocksize;
}

/*
 * Return the capacity of the logical device.
 * On return "blk" has the blocksize of the
 * device
 */
int readcap( int fd, int *blk )
{
  int lba = 0;

  sg_io_hdr_t buf;

  unsigned char data[8];
  char sbp[32];

  char cmd[10] = { READ_CAPACITY, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

  /*
   * Construct the sg_io_hdr structure for a READ_CAPACITY command
   */
  memset( &buf, 0, sizeof( sg_io_hdr_t ) );
  buf.interface_id = 'S';
  buf.dxfer_direction = SG_DXFER_FROM_DEV;
  buf.cmd_len = 10;
  buf.mx_sb_len = 32;
  buf.iovec_count = 0;
  buf.dxfer_len = 8;
  buf.dxferp = data;
  buf.cmdp = cmd;
  buf.sbp = sbp;
  buf.timeout = 3000;
  buf.flags = 0;

  /*
   * Perform the READ/WRITE
   */
  if( ioctl( fd, SG_IO, &buf ) < 0 )
  {
    printf( "Unable to read capacity of device.\n" );
    perror( "ioctl()" );
    close( fd );
    removeModule( module );
    exit(EXIT_FAILURE);
  }

  /*
   * Construct the last logical block
   */
  lba |= data[3];
  lba |= data[2] << 8 ;
  lba |= data[1] << 16;
  lba |= data[0] << 24;

  /*
   * Construct the logical block size
   */
  *blk = 0;
  *blk |= data[7];
  *blk |= data[6] << 8;
  *blk |= data[5] << 16;
  *blk |= data[4] << 24;

  if( debug ) printf( "Capacity of device = %d logical blocks, each of size %d\n\n",
    lba, *blk );

  close(fd);

  return lba;
}
